@@ -2,7 +2,7 @@ package ServiceNow::Simple;
 use strict;
 use warnings FATAL => 'all';
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 use Data::Dumper;
 use FindBin;
@@ -12,9 +12,12 @@ use LWP::UserAgent;
 use SOAP::Lite;
 use XML::Simple;
 use Carp;
+use File::Basename;
+use MIME::Types;
+use MIME::Base64 ();
 
-$Data::Dumper::Indent=1;
-$Data::Dumper::Sortkeys=1;
+$Data::Dumper::Indent   = 1;
+$Data::Dumper::Sortkeys = 1;
 
 our %config;
 my $user;
@@ -398,12 +401,12 @@ sub insert
 }
 
 
-
 sub soap_debug
 {
     SOAP::Lite->import(+trace => 'all');
 }
 
+
 sub print_results
 {
     my ($self, $flag) = @_;
@@ -434,6 +437,12 @@ sub set_table
     }
 }
 
+sub _get_table
+{
+    my $self = shift;
+
+    return $self->{table};
+}
 
 sub set_instance
 {
@@ -663,6 +672,85 @@ sub set_plus_display_value
 }
 
 
+sub _get_file_base64
+{
+    my ($self, $filename) = @_;
+    my $fh;
+    unless (open($fh, '<', $filename))
+    {
+        carp("Failed to open file $filename for reading $!\n");
+        return;
+    }
+
+    # If you want to encode a large file, you should encode it in chunks that are a multiple of 57 bytes.
+    # This ensures that the base64 lines line up and that you do not end up with padding in the middle.
+    # 57 bytes of data fills one complete base64 line (76 == 57*4/3):
+    my $base64;
+    my $bytes;
+    while (read($fh, $bytes, 60*57))
+    {
+        $base64 .= MIME::Base64::encode($bytes);
+    }
+    close $fh;
+
+    return $base64;
+}
+
+
+sub add_attachment
+{
+    my ($self, $args_h) = @_;
+
+    my $method = $self->_get_method('insert');
+
+    # The required arguments
+    my $file_name = $args_h->{file_name};
+    my $table     = $args_h->{table};
+    my $sys_id    = $args_h->{sys_id};
+
+    carp("Required parameters are file_name, table and sys_id\n") unless ($file_name && $table && $sys_id);
+
+    my ($name, $path, $suffix) = fileparse($file_name, '\.[^.]+$');
+
+    my $mt   = MIME::Types->new;
+    my $type = $mt->mimeTypeOf($suffix);
+
+    # Make sure we have a type
+    $type = 'application/octet-stream' unless $type;
+
+    my $base64 = $self->_get_file_base64($file_name);
+
+    # Change to the correct table, storing the previous so we can reinstate it
+    my $prev_table = $self->_get_table();
+    $self->set_table('ecc_queue');
+
+    my %params = (
+        agent   => 'ServiceNow::Simple',
+        topic   => 'AttachmentCreator',                   # Required
+        name    => join(':', $name . $suffix, $type),   # File name and required MIME type
+        source  => join(':', $table, $sys_id),            # The table and sys_id of the record the attachment relates to
+        payload => $base64,                               # The base64 encoded file
+        );
+    #print STDERR Dumper(\%params), "\n";
+    my @params = $self->_load_args(\%params);
+    my $result = $self->{soap}->call($method => @params);
+
+    # Print faults to log file or stderr
+    $self->_print_fault($result);
+
+    # insertResponse
+    if ($result && $result->body && $result->body->{insertResponse})
+    {
+        return $result->body->{insertResponse};
+    }
+
+    # Restore to the previous table, so no one knows we were here :)
+    $self->set_table($prev_table);
+
+    return;
+}
+
+
 sub _init
 {
     my ($self, $args) = @_;
@@ -1242,6 +1330,18 @@ This array can be used to select the data, print a header, act as keys to set he
  }
 
 
+=head2 add_attachment
+
+Add a attachment (file) to a record in a table.
+
+Example:
+
+ $result = $sn->add_attachment({
+             file_name => '/tmp/my_sample.xls',
+             table     => 'incident',
+             sys_id    => '97415d1f1903ac00fb54adb1ad53dc1a',
+             });
+
 =head2 insert
 
 Creates a new record for the table.  B<Note:> You need to provide all fields configured
@@ -1451,7 +1551,7 @@ password and they are not visible in a whole bunch of scripts.
 
 =head1 VERSION
 
-Version 0.09
+Version 0.10
 
 =cut
 
